package cn.iocoder.yudao.module.reward.api.reward;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.iocoder.yudao.framework.redis.helper.RedisDistributedLock;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.api.user.MemberUserApi;
import cn.iocoder.yudao.module.member.api.user.dto.UserRespDTO;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemRewardReqVO;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemRewardRespDTO;
import cn.iocoder.yudao.module.reward.dal.dataobject.redeem.RewardRedeemDO;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.module.reward.dal.mysql.redeem.RewardRedeemMapper;
import cn.iocoder.yudao.module.reward.dal.mysql.reward.RewardMapper;
import cn.iocoder.yudao.module.reward.enums.RewardRedeemStatusEnum;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.GET_LOCK_FAILED;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.USER_NOT_EXISTS;
import static cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants.*;

@Service
@Validated
public class RedeemRewardApiImpl implements RedeemRewardApi {

    @Resource
    private RewardMapper rewardMapper;
    @Resource
    private RewardRedeemMapper rewardRedeemMapper;
    @Resource
    private MemberUserApi memberUserApi;
    @Resource
    private RedisDistributedLock redisDistributedLock;
    @Resource
    private MemberUserScoreApi memberUserScoreApi;
    @Resource
    private SnowflakeGenerator snowflakeGenerator;
    @Override
    public RedeemRewardRespDTO redeemReward(RedeemRewardReqVO redeemRewardReqVO) {

        boolean lock = false;
        try {
            lock = redisDistributedLock.lock("reward:redeem:lock:" + redeemRewardReqVO.getRewardId());
            if (!lock) {
                throw exception(GET_LOCK_FAILED);
            }
            // 查询会员积分
            UserRespDTO memberUser = memberUserApi.getUser(redeemRewardReqVO.getRewardId());
            if (memberUser == null) {
                throw exception(USER_NOT_EXISTS);
            }
            Integer holdScore = memberUser.getHoldScore();

            // 查询礼品
            RewardDO rewardDO = rewardMapper.selectById(redeemRewardReqVO.getRewardId());
            if (rewardDO == null) {
                throw exception(REWARD_NOT_EXISTS);
            }
            // 礼品未开启
            if (rewardDO.getStatus() != 1) {
                throw exception(REWARD_NOT_ENABLE);
            }
            // 会员积分不够
            if (holdScore < rewardDO.getPointsRequire() * redeemRewardReqVO.getRewardCount()) {
                throw exception(REWARD_SCORE_NOT_ENOUGH);
            }
            // 礼品数量不够
            if (rewardDO.getQuantityRemain() < redeemRewardReqVO.getRewardCount()) {
                throw exception(REWARD_COUNT_NOT_ENOUGH);
            }
            // 兑换方式不匹配
            if (!Objects.equals(rewardDO.getPickMethod(), redeemRewardReqVO.getRedeemType())) {
                throw exception(REWARD_PICK_METHOD_NOT_ALLOW_CREATE);
            }
            //TODO：校验兑换次数（查兑换记录）

            // 更新礼品
            rewardMapper.updateById(rewardDO);

            // 添加兑换记录
            Long redeemId = addRedeemRecord(redeemRewardReqVO);
            // 更新会员积分
            updateMemberScore(redeemRewardReqVO, rewardDO, redeemId);

        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            redisDistributedLock.releaseLock("reward:redeem:lock:" + redeemRewardReqVO.getRewardId());
        }
        return null;
    }

    private void updateMemberScore(RedeemRewardReqVO redeemRewardReqVO, RewardDO rewardDO, Long redeemId) {
        Map<String, Object> extParam = new HashMap<>();
        extParam.put("redeemId", redeemId);
        memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                        .memberId(redeemRewardReqVO.getMemberId())
                        .sourceType(ScoreSourceTypeEnum.EXCHANGE_REWARD)
                        .scoreCount(redeemRewardReqVO.getRewardCount()* rewardDO.getPointsRequire())
                        .extParam(extParam)
                .build());
    }

    private Long addRedeemRecord(RedeemRewardReqVO redeemRewardReqVO) {
        RewardRedeemDO rewardRedeemDO = BeanUtil.copyProperties(redeemRewardReqVO, RewardRedeemDO.class);
        rewardRedeemDO.setId(snowflakeGenerator.next());
        rewardRedeemDO.setStatus(RewardRedeemStatusEnum.REDEEMING.getValue());
        rewardRedeemMapper.insert(rewardRedeemDO);
        return rewardRedeemDO.getId();
    }

    @Override
    public List<RedeemRewardRespDTO> redeemRewards(List<RedeemRewardReqVO> redeemRewardReqVOList) {
        Long rewardId = redeemRewardReqVOList.get(0).getRewardId();
        // 查询礼品
        RewardDO rewardDO = rewardMapper.selectById(rewardId);
        // 礼品未开启
        if (rewardDO.getStatus() != 1) {
            throw exception(REWARD_NOT_ENABLE);
        }
        boolean lock = false;
        try {
            lock = redisDistributedLock.lock("reward:redeem:lock:" + rewardId);
            if (!lock) {
                throw exception(GET_LOCK_FAILED);
            }
            // 校验礼品数量
            //
            ArrayList<Long> memberIds = new ArrayList<>();
            int totalCount = 0;
            ArrayList<UserRespDTO> members = new ArrayList<>();
            for (RedeemRewardReqVO redeemRewardReqVO : redeemRewardReqVOList) {
                // 每个兑换VO校验一遍
                // 兑换方式不匹配
                if (!Objects.equals(rewardDO.getPickMethod(), redeemRewardReqVO.getRedeemType())) {
                    throw exception(REWARD_PICK_METHOD_NOT_ALLOW_CREATE);
                }
                // 查询会员id列表，暂时挨个查
                memberIds.add(redeemRewardReqVO.getMemberId());
                members.add(memberUserApi.getUser(redeemRewardReqVO.getMemberId()));
                // 记录兑换总数
                totalCount+=redeemRewardReqVO.getRewardCount();
            }
            // 查询会员列表，获取积分列表

            // 判断兑换总数是否大于礼物数量
            if (totalCount > rewardDO.getQuantityRemain()){
                throw exception(REWARD_COUNT_NOT_ENOUGH);
            }
            // 判断会员积分是否够（初步校验，实际兑换时会再加锁校验）


        } catch (Exception e) {
            throw exception(REWARD_REDEEM_FAIL);
        }
        finally {
            redisDistributedLock.releaseLock("reward:redeem:lock:" + rewardId);
        }
        return null;
    }
}
